{% extends "appbuilder/base.html" %}
{% import 'appbuilder/general/lib.html' as lib %}

{% block content %}
{{ lib.panel_begin(title) }}

{% block add_form %}
    {{ widgets.get('add')(form_action=form_action)|safe }}
{% endblock %}

{{ lib.panel_end() }}
{% endblock %}
